/*	----------------------------------------------------------------------------------------------------------------------------------

	Open4D.h
	C Interface file for Macintosh and Windows

	Original design by	Laurent Ribardire and Asmae Benkirane

	Copyrights 1994-98 ACI, ACI US Inc. - All rights reserved
		
	---------------------------------------------------------------------------------------------------------------------------------
	I	Modifications History																																																					I
	---------------------------------------------------------------------------------------------------------------------------------
	I	Date 					I	Versions								I	Modifications																											I
	---------------------------------------------------------------------------------------------------------------------------------
	I	01/25/94				I	4D Open 1.0	F1							I	Synchronous and Asynchronous Mode have been added									I
	I							I	4D Server 1.1							I																																		I
	I							I	ADSP NetworkComponent 1.1.2				I																																		I
	I							I	TCP/IP NetworkComponent 1.0.1			I																																		I
	---------------------------------------------------------------------------------------------------------------------------------
	I 02/10/94					I	4D Server 1.1.1							I	_4D_GotoRECORD has been added																			I
	---------------------------------------------------------------------------------------------------------------------------------
	I 09/03/94					I	p2c et PPC								I	_4D_GotoRECORD has been added																			I
	---------------------------------------------------------------------------------------------------------------------------------
	I 11/11/95					I	M68k & MPPC & WIN32						I	Entirely redisign by L. Voillot																			I
	---------------------------------------------------------------------------------------------------------------------------------
	I 07/26/96					I	added _4D_Select4DServerWithOther		I	Loic Vandereyken 																			I
	---------------------------------------------------------------------------------------------------------------------------------
	I 7/15/97					I	bug fixes, code changes, adapt to V6	I	Jerry Hale																			I
	---------------------------------------------------------------------------------------------------------------------------------
	I 3/2/98					I	bug fixes								I
	
-----------------------------------------------------------------------------------------------------------------------------------*/
	
	
#ifndef __4D_OPEN__
#define __4D_OPEN__
#ifndef OpenLib
#define OpenLib 1
#endif

//_________________________________________________________________
//	Constants which are returned by _4D_GetVersionNumber
//	DO NOT ALTER THESE VALUES
//	kMajorVers is returned in the High Word
#define kMajorVers 2004
//	kBuildNum is returned in the Low Word
#define kBuildNum 1

//_________________________________________________________________
// we define here the platform used (FORMAC, FORPPC, FORWIN)
#if WINVER
	#define FORWIN 1
	#define FORPPC 0
	#define FORMAC 0
	#define MACVER 0	
#else
	#define MACVER 1
#if defined(powerc) || defined (__powerc)
	#define FORPPC 1
	#define FORWIN 0
	#define FORMAC 0
#endif
#endif

#if !FORWIN && !FORPPC
	#define FORMAC 1
#endif
#if !MACVER && !WINVER
#error "either MACVER or WINVER must be set"
#endif

#if OpenLib

#if MACVER
#if __MWERKS__ //|| PRAGMA_ALIGN_SUPPORTED //CD 15/01/2001, unused now
#pragma options align=mac68k
#endif
// by setting XPlatform to one, we make sure this code will compile on other platforms
// than macintosh (ie. windows)
// for backward compatibility defaults to 0
#if XPlatform
#define MacintoshStyleHandle 0
#else
#define MacintoshStyleHandle 1
#endif
#endif

#if WINVER
#define MacintoshStyleHandle 0
#endif

#else
#define MacintoshStyleHandle 1
#endif

#if WINVER
	#if OpenLib
		#define true 1
		#define false 0
		#define noErr 0
		#include "Windows.h"
		#define powerc
	#else
		#include "ToolBox.h"
		#define ToolBox_d
	#endif
	#pragma pack(2)
	#define ST_PASCAL( thetype ) thetype pascal 
#endif
#if MACVER
	#if !TARGET_RT_MAC_MACHO
		#include <Types.h>
		#include <CodeFragments.h>
	#endif
	#define ST_PASCAL( thetype ) pascal thetype
	#define ToolBox_d
#endif

#ifdef __cplusplus
	extern "C" {
#endif

#define CURNUMVERS 0xF0010606
#define MinServer  0xA00F0525

// note: these constants should not be used
// but you should handle multiple protocols	gracefully
#define NC_ADSP			1		//	Macintosh and Windows environments
#define NC_IPX			4		//	Macintosh and Windows environments
#if WINVER
#define NC_TCPIP		2		//	Windows environment
#endif
#if MACVER
#define NC_TCPIP	   29		//	Macintosh environment
#define NC_MEM			69		//	Macintosh environment

#endif

enum {
/*		- 4D Server errors codes --------------------------------------------------------------------------------------------------------  */
ErrUserIsNotInGroup				=	-9944,	/* The user does not belong to 4D Open Group  */
ErrNamedSelNotExist				=	-9946,	/* Unable to clear the named selection because it does not exist */
ErrOnly4DClientConnection = -9947,	/* The Allow 4D Client connections only check box has been selected */
ErrBadDataSegmentNumber		=	-9950,  /* Invalid data segment number */
ErrNotRelatedField				= -9951,  /* Field is not related to another one */
ErrBadDataSegmentHeader		= -9952,	/* Invalid data segment */
ErrNoLogFile							=	-9953,	/* There is no log file */
ErrNoCurrentRECORD				= -9954,	/* there is no current RECORD */
ErrQTNotInstalled					=	-9955,	/* QuickTime is not installed */
ErrBadVersion							= -9956,  /* 4D Open version not compatible with 4D Server */			
ErrListIsLocked						= -9957,  /* The list is locked */
ErrProcessNotStarted			= -9958,  /* Process cannot be started */
ErrBackupAlreadyRunning		= -9959,  /* The backup process has already been started by another user or process */
ErrBackupNotInstalled			= -9960,  /* 4D Backup is not installed on the server */
ErrBackupNotRunning				= -9961,  /* The backup process is not currently running */
ErrBackupCannotLaunched		= -9962,  /* The backup cannot be launched because the server is shutting down */
ErrBadRECORDNumber				= -9963,  /* Invalid RECORD number requested */
ErrBadSortDefinition			= -9964,  /* Bad sort definition table sent */
ErrBadSearchDefinition		= -9965,  /* Bad search definition table sent */
ErrIncompatibleTypes			= -9966,  /* Invalid type requested */
ErrRECORDNotLoaded				= -9967,  /* The RECORD was not modified because it could not be loaded */
ErrBadSelectedRECORDNumber		= -9968,  /* Invalid selected RECORD number requested */
ErrBadFieldType						= -9969,  /* Invalid field type requested */
ErrFieldNotIndexed				= -9970,  /* Field is not indexed */
ErrBadFieldNumber					= -9971,  /* Field number is out of range */
ErrBadFileNumber					= -9972,  /* File number is out of range */
ErrBadTRIC								= -9973,  /* The TRIC resources are not the same */
ErrRECORDAlreadyDeleted		= -9974,  /* The RECORD has already been deleted */
ErrCannotLoadTrIdxPage		= -9975,  /* Transaction index page could not be loaded */
ErrBackupInProgress				= -9976,  /* Backup in progress, no modifications are allowed */
ErrBadSelection						= -9977,  /* The selection does not exist */
ErrBadPassword						= -9978,  /* Invalid password */
ErrBadUserName						= -9979,  /* Unknown user */
ErrStructureIsLocked			= -9980,  /* The file cannot be created because the structure is locked  */
ErrBadFileFieldTable			= -9981,  /* Invalid Field/File number definition table sent */
ErrRECORDNotInSelection		= -9982,  /* The RECORD cannot be loaded because it is not in the current selection */
ErrDuplicateKeyInTrans		= -9984,  /* Transaction has been cancelled because of a duplicated index */
ErrRecursiveIntegrity			= -9985,  /* Recursive integrity */
ErrRECORDLockedDuringDel	= -9986,  /* RECORD locked during an automatic deletion action */
ErrRelatedRECORDs					= -9987,  /* Other RECORDs are already related to this RECORD */
ErrInvalidStruture				= -9989,  /* Invalid structure  (database needs to be repaired) */
ErrTimeOut								= -9990,  /* Time-out error */
ErrNoEnoughPrivilege			= -9991,  /* Access privileges error */
ErrCommInterruptedByUser	= -9994,  /* Serial communication interrupted by the user */
ErrDemoVersionLimit				= -9995,  /* Limited demonstration version */
ErrStackIsFull						= -9996,  /* Stack is full */
ErrMaxNumberOfRECORDs			= -9997,  /* Maximum number of RECORDs has been reached */
ErrDuplicateKeyIndex			= -9998,  /* Duplicate index key */
ErrNoMoreSpaceToSave			= -9999,  /* No more space available to save the RECORD */
ErrBadRECORDAdress				= -10500,	/* Invalid RECORD address */
ErrInvalidIdxPage					= -10501,	/* Invalid index page (index needs to be repaired) */
ErrInvalidRecStruct				= -10502,	/* Invalid RECORD structure (data file needs to be repaired) */
ErrRecOutOfRange					= -10503,	/* RECORD is out of range */
ErrIdxPgOutOfRange				= -10504,	/* Index page is out of range */

/* 	- Network Components errors codes -----------------------------------------------------------------------------------------------  */

ErrBadState 							= -10001,		/*The actual connection to the database has been disrupted */
ErrTearDown 							= -10002,		/*The connection for this process has been disrupted*/
ErrBadParam 							= -10003,		/* Bad connection parameters */
	
ErrNoServerSelected 			= -10020,		/* No server was selected while using _4D_Select4DServer */
ErrServerNotFound 				= -10021,		/* No server was found while using _4D_Find4DServer */
ErrUnknownOption 					= -10050,		/* Unknown option in Get/SetOption (unused in this version) */
ErrBadParamValue 					= -10051,		/* Incorrect value in Get/SetOption (unused in this version) */

/* - 4D Open errors codes ---------------------------------------------------------------------------------------------------------- */ 
	
ErrReqInProcess 					= -32001, 	/* A query in asynchronous mode is already in progress */
ErrLockedRecInSel					= -32002, 	/* Warning, there some locked RECORDs in your selection, after calling _4D_DeleteSelectedRecords and _4D_ArrayToSelection  */
ErrConnectNotValid				= -32003,	 	/* CIDH = nil, Invalid connection handle */

ErrMemory						= -32004,

ErrNoRASSoftware				= -10052,
ErrRASConnectionCancelled		= -10053,
ErrNoMoreThanOneConnection		= -10054

};

/*
- Description des erreurs en franais -------------------------------------------------------------------------------------------

	Codes d'erreurs retourns par 4D Server

	-9944			L'utilisateur n'appartient pas au groupe 4D Open
	-9946			Impossible de supprimer la slection temporaire car elle n'existe pas
	-9947			L'option N'autorisez que les connexions 4D Client est slectionne
	-9950			Mauvais numro de segment de donnes
	-9951			La rubrique n'est pas lie
	-9952			Le numro de segment de donnes est invalide
	-9953			Il n'y a pas de fichier d'historique
	-9954			Il n'y a pas de fiche courante
	-9955			QuickTime n'est pas install
	-9956			La version de 4D Open ne corresponds pas  celle de 4D Server
	-9957			L'numeration est verrouille 
	-9958			La tche ne peut pas tre dmarre 
	-9959			La tche 4D Backup a t dmarre par un autre utilisateur
	-9960			Le module 4D Backup module n'est pas install sur la machine serveur 
	-9961			La tche 4D Backup n'a pas t dmarre 
	-9962			Le backup ne peut pas tre dmarr maintenant car le serveur est train de quitter 
	-9963			Numro de fiche invalide 
	-9964			Mauvaise formulation de tri 
	-9965			Mauvaise formulation de recherche 
	-9966			Les types de donnes ne correspondent pas 
	-9967			La fiche ne peut pas tre modifie car elle n'a pas p tre charge 
	-9968			Numro de fiche hors slection 
	-9969			Type de rubriques incorrect 
	-9970			La rubrique n'est pas indexe 
	-9971			Le numro de rubrique est hors intervalle 
	-9972			Le numro de fichier est hors intervalle
	-9973			Les ressources TRIC ne sont pas compatibles 
	-9974			Fiche dj supprime 
	-9975			La page d'index de transaction n'a pas p tre charge 
	-9976			Backup en cours, les modifications ne sont pas permises durant cette opration
	-9977			Slection inexistante 
	-9978			Mauvais mot de passe 
	-9979			Utilisateur inconnu 
	-9980			Le fichier ne peut pas tre ajout car la structure de la base est pour l'instant verrouille, ressayez plus tard
	-9981			Liste de couple de noms incorrecte 
	-9982			La fiche ne peut pas tre charge car elle n'est pas dans la slection courante de la machine client
	-9984			Transaction annule en raison d'une erreur de cl d'index unique duplique
	-9985			Un contrle d'intgrit rcursif a t dtect 
	-9986			Une ou des fiches verrouilles ont t rencontres lors d'une suppression automatique de fiches 
	-9987			Cette fiche est lie  d'autres fiches 
	-9989			La structure de la base est abme 
	-9990			Erreur de Time-out 
	-9991			Erreur de privilge d'accs
	-9994			Interruption dclenche par l'utilisateur 
	-9995			La limite de la version dmo de 4D Serveur a t rencontre
	-9996			La pile (stack) est pleine 
	-9997			Le nombre maximum de fiches a t atteint 
	-9998			Cl d'index duplique 
	-9999			Plus de place sur le disque pour enregistrer la fiche 
	-10000		Adresse de fiche invalide 
	-10001		Page d'index invalide (l'index doit tre recr) 
	-10002		Fiche invalide (le fichier de donnes doit tre rpar) 
	-10003		La fiche est hors limites 
	-10004		La page d'index est hors limites 
	
	Codes d'erreurs retourns par les composants rseaux
				
	-10001		La connexion physique ou logicielle au 4D Server a t interrompue 
	-10002		Cette connexion particulire a t interrompue
	-10003		Mauvais paramtres de connexion 
					
	-10020:		Aucun 4D Server n'a t choisi dans le dialogue de connextion prsent par _4D_Select4DServer 
	-10021:		Le 4D Server n'a pas t trouv par _4D_Find4DServer sur le rseau 
						ou le(s) fichier(s) d'interface contienne(nt) une description de serveur incorrecte
	-10050		L'option passe  Get/SetOption est inconnue
	-10051		La valeur passe  Get/SetOption est incorrecte
	
	Codes d'erreurs retourns par 4D Open

	-32001		Tentative d'envoi de requte au travers d'une connexion excutant une requte asynchrone
	-32002		Une ou plusieurs fiches verrouilles ont t rencontrs par _4D_DeleteSelectedRecords ou _4D_ArrayToSelection
	-32003		La handle de la connexion est invalue (CIDH = NIL)
*/



#ifndef __OPEN4D_NC__
/*	The following 4D OPEN data types are common to some Open4DNC data types. */
			
	typedef short			int2;
	typedef long 			int4;
	typedef unsigned short 	uint2;
	typedef unsigned long 	uint4;
	typedef unsigned char 	byte;
	typedef unsigned char 	*pstring;
	typedef unsigned char 	st15[16];					
	typedef unsigned char 	st32[32];
	typedef unsigned char 	st80[82];
	
	#if WINVER && OpenLib
		typedef void *				Handle4D;
		typedef int4				OSType ;
		typedef unsigned char		uchar ;
		typedef struct Rect4D {  
		    int2 left;       
		    int2 top;        
		    int2 right;      
		    int2 bottom;     
		} Rect4D;
	#else
		typedef Rect			Rect4D;
		typedef Handle			Handle4D;
	#endif
#endif				/* end of __OPEN4D_NC__ */

/*  ---------------------------  */

typedef struct IDRec {
	 st32 	StationName;
	 st32 	UserName;
	 st32 	UserPass;
	 st32 	TaskName;
	 byte 	isapi;
	 byte 	unused;
} IDRec;
						

/* ----------------------------------- */

typedef struct  reqdata {
				int2		numpack;		// ID of the external procedure runing on server
				int4		typ;			// your own selector
				int2		num;			//
				int2		data;			// 
}reqdata;
typedef reqdata *reqdataPtr;

/* ----------------------------------- */


#ifndef __EXTERNAL4D__
	/*	The following 4D OPEN data types are common to some 4D External Kit data types. 
			If you want to be able to compile a project including both 4D Open and 4D External,
			add the line :
						#ifndef __EXTERNAL4D__
						#define __EXTERNAL4D__
			to your 4D External interface file or to a file of your project. */
			
typedef struct Date4D {
	int2 	Day;
	int2 	Month;
	int2	Year;
} Date4D;

typedef  Date4D 	*Date4DPtr;

typedef struct TE4D {									/* 4th DIMENSION Text */
	int2 		TextLen;
	Handle4D	hText;
}TE4D;

typedef TE4D *TE4DPtr;

typedef struct Pict4D{
	int4		PictLen;
	Handle4D	hPict;
}Pict4D;

typedef  Pict4D *Pict4DPtr;

typedef struct Blob4D {
	int4		BlobLen;
	Handle4D	hBlob;
} Blob4D;

typedef  Blob4D	*Blob4DPtr;

#if defined(powerc) || WINVER
typedef double NativeReal;
#else
typedef extended NativeReal;
#endif

typedef	union Real {
	NativeReal		r;
	unsigned char	c[10];
	unsigned short	w[5];
	unsigned long	l[2];
} Real;
		
		
// for compatibility with the pascal headers (where real is a reserved word)
// Real4D can be used in place of Real
typedef Real Real4D; 

enum {
	_noConv_ = 0 ,
	_8to10_ ,
	_10to8_
} ;

/********************** Types of Fields (types des rubriques) **********************/

#define		s_AlphaNumeric		0				/*  Alphanumeric field  */
#define		s_Number			1				/*  Real field  */
#define		s_Text32K			2				/*  Text field  */
#define		s_Pictures			3				/*  Picture field  */
#define		s_Dates				4				/*  Date field  */
#define		s_Bools				6				/*  Boolean field  */
#define		s_SubFile			7				/*  Subfield field  */
#define		s_Int16				8				/*  INTEGER field  */
#define		s_Int32				9				/*  int4Int field  */
#define		s_TimeSec			11				/*  Time field  */
#define		s_Blob				30				/*  Blob field  */

#define		None2				0
#define		Logic_And			1
#define		Logic_Or			2
#define		Except				3

#define		None1				0
#define		Equal				1
#define		Diff				2
#define		Greater_than		3
#define		Greater_or_Equal	4
#define		Less_than			5
#define		Less_or_Equal		6
#define		Contains			7
#define		Does_not_contain	8

#endif				/* end of __EXTERNAL4D__ */
					
typedef byte DataType ;
				 
typedef struct DataRec {
					DataType 		typ;
					byte 			filler;
					union {
							TE4D 		text; 		/* s_Text32K */
							st80 		s;	 		/* s_AlphaNumeric */
							Real 		r; 			/* s_Number  */
							int2 		b;  		/* s_Bool    */
							int2 		i;  		/* s_Int16   */
							int4 		l;  		/* s_Int32   */
							int4 		tim;		/* s_TimeSec */
							Date4D 		d; 			/* s_Dates   */
							Pict4D 		pic; 		/* s_Pictures */
							Blob4D		blob;		/* s_Blob	*/
							struct { st80 s2 ; int2 unused; } other ;
						} u;
					} DataRec;

typedef struct SearchLine {
						int2 			Field_Number;
						int2 			File_Number;
						byte 			SOP;		/* search operator */
						byte 			unused;
						DataRec 		Value;
						byte 			LOP;   	/* logical operator */
						byte 			unused2,unused3 ;
					} SearchLine;
					
typedef struct SearchRecord {
						int2 				NB_Lines;
						int2 				TargetFile;
						SearchLine 			lines[1];
					} SearchRecord;
					
typedef SearchRecord *SearchRecordPtr;

/* ----------------------------------- */
						
typedef struct SortLine {
						int2 			Field_Number;
						int2 			File_Number;
						byte 			Ascent;
					} SortLine;
					
typedef struct SortRecord {
						int2 			NB_Lines;
						int2 			TargetFile;
						SortLine 		lines[1];
					} SortRecord;
					
typedef SortRecord *SortRecordPtr;

/* ----------------------------------- */
					
typedef struct ReqFieldRec {
						int2 		NB_Fields;
						int2 		TargetFile;
						int2 		Fields[1];
					} ReqFieldRec;
					
typedef ReqFieldRec *ReqFieldRecPtr;
#if !MacintoshStyleHandle
typedef Handle4D ReqFieldRecHandle;
#else
typedef ReqFieldRecPtr *ReqFieldRecHandle;
#endif

typedef struct NameCouple {
						st32 		FileName;
						st32 		FieldName;
					} NameCouple;
					
typedef struct ReqFieldNames {
						int2 		NB_NameCouples;
						NameCouple 	Names[1];
					} ReqFieldNames;
						
typedef ReqFieldNames *ReqFieldNamesPtr;
						
/* ----------------------------------- */
/* FieldAttributes */			

#define field_Indexed 						0x8000
#define field_Unique 						0x4000
#define field_Mandatory 					0x2000
#define field_HasAList 						0x1000
#define field_Modifiable 					0x0800
#define field_Enterable 					0x0400
#define field_Invisible 					0x0100
#define field_AutomaticManyToOne 			0x0040
#define field_AutomaticOneToMany 			0x0020
#define field_RelationalIntegrity 			0x0004
#define field_RejectDelete 					0x0002
#define field_UpdateLinkedValue 			0x0001

/* FieldAttributes for Pict Field type */			

#define	pict_Compressed  0x8000
#define	pict_OnDiskOnly  0x4000

typedef int2 FieldAttributes;

/* PicAttributes is unused right now */
typedef int2 PicAttributes;

typedef struct FieldDef {
						st32 	Name; /* so that v6 works correctly */
						byte 	Typ,
								Filler;
						int2 	AlphaLen;
						int2 	unused1,
								unused2;
						union {
							struct {
									int2 							RelatedFile;
									int2 							RelatedField;
									int2 							IndexNumber;
									int2 							unused;
									FieldAttributes 				Att;
									int2 							NB_Sublevels;
									int2 							SubLevels[4];
									int2							unused2;
									int2 							DiscriminantField;
									int2							unused3;
								} SimpleField;
							struct {
									int2 							unused[6];
									PicAttributes 					PicAtt;
									int2 							CompressDriver;
								} PictField;
							struct {
									int4 							unused,unused2;
									int2 							SubFileRef;
									byte 							unused3;
								} SubFile;
						} u;
					} FieldDef;

typedef FieldDef *FieldDefPtr;
#if !MacintoshStyleHandle
typedef Handle4D FieldDefHandle;
#else
typedef FieldDefPtr *FieldDefHandle;
#endif

typedef struct FileDef {
						int2 			NB_Fields;
						int2 			Invisible;
						int2			unused1;
						int2			FileNum;
						int2 			unused2;
						int4			unused4;
						Rect4D 			Frame;
						int2 			unused3;
				        char            TrigOnSavingNew;
				        char            TrigOnSavingExisting;
				        char            TrigOnDeleting;
				        char            TrigOnLoading;
						int2			unused5;
						FieldDef 		Fields[1];
					} FileDef;
						
typedef FileDef *FileDefPtr;
#if !MacintoshStyleHandle
typedef Handle4D FileDefHandle;
#else
typedef FileDefPtr *FileDefHandle;
#endif

/* ----------------------------------- */

typedef struct ProcessInfo {
						st32 		UserName;
						st32 		StationName;
						st32 		Taskname;
						int4 		TotalTimeSpent;
						int2 		State;
					} ProcessInfo;
					
typedef struct ListProcess {
						int2 			NB_Process;
						ProcessInfo 	Processes[1];
					} ListProcess;
					
typedef struct ListProcess *ListProcessPtr;
#if !MacintoshStyleHandle
typedef Handle4D ListProcessHandle;
#else
typedef ListProcessPtr *ListProcessHandle;
#endif

/* ----------------------------------- */

typedef struct UserRec {
						int4 			unused;
						st32 			UserName;											
						st15 			unused1;	
						st32 			ProcToCall;
						Rect4D 			unused2,unused3;	
						int2 			LoginCount;
						Date4D 			LastLogin;
						int4 			DefaultOwner;
					} UserRec;
					
typedef struct GroupRec {
						st32 			GroupName;
						int2 			UserCount;
						int2 			Owner;
						int2 			Users[1];
					} GroupRec;
					
typedef GroupRec *GroupRecPtr;
#if !MacintoshStyleHandle
typedef Handle4D GroupRecHandle;
#else
typedef GroupRecPtr *GroupRecHandle;
#endif

/* ----------------------------------- */

typedef struct {
	int2 nbFiles;
	st32 fileNames[1];
} FileList;

typedef 	FileList *FileListPtr;
#if !MacintoshStyleHandle
typedef Handle4D FileListHandle;
#else
typedef FileListPtr *FileListHandle;
#endif
/* ----------------------------------- */
						
typedef struct ServerInfoRec {
	int4		ServerCurVers;				/* Current Serveur version */
	byte		ServerHasInvertedData;
} ServerInfoRec;

/* ------------------------------------ */

typedef struct VariableDefRec {
	st32		Name;
	int4		Index1,
				Index2;
	byte		Attr,
				Reserved2;
	unsigned long StackStamp;
} VariableDefRec, *VarDefPtr;

typedef struct ArrayBlock{
	int4		Size;
	Handle4D	Data;
	int2		Current;
}ArrayBlock, *ArrayPtr;

typedef struct ValueBlock {
    byte	Type,Filler;
    union {
        Real		Real;
        Date4D		Date;
        int4		Time;
        int2		Bool;
        int2		Integer;
        int4		Longint;
        TE4D		Text;
		Pict4D 		Picture;
		Blob4D		Blob;
		byte		String[256];
		ArrayBlock	Array;
		byte		Reserved1[82];
    } u;
} ValueBlock, *ValuePtr;

typedef struct ProcessVarRec {
	VariableDefRec	VarDef;
	ValueBlock		Value;
} ProcessVarRec, *ProcessVarRecPtr;

typedef struct ServerExecRec {
	st32			ProcName,
					ProcessName;
	int4			StackSize,
					Time;
	int2			ParmCount;
	byte			Unique,
					Filler;
	ProcessVarRec	Param[1];
} ServerExecRec, *ServerExecPtr;

#if USEWINDOWSHANDLE
	typedef DXHANDLE ServerExecHandle;
#else
	typedef ServerExecPtr *ServerExecHandle;
#endif

/* ------------------------------------ */

enum { kLocalVar, kProcessVar, kInterProcessVar };


typedef struct GetProcVarBlock {
	int2				ProcNum,
						ParmCount;
	ProcessVarRec		Param[1];
} GetProcVarBlock, *GetVarBlkPtr;

typedef struct SetProcVarBlock {
	int2				ProcNum,
						ParmCount;
	ProcessVarRec		Param[1];
} SetProcVarBlock, *SetVarBlkPtr;

#if !MacintoshStyleHandle
	typedef Handle4D SetVarBlkHandle;
#else
	typedef SetVarBlkPtr *SetVarBlkHandle;
#endif

#if !MacintoshStyleHandle
	typedef Handle4D GetVarBlkHandle;
#else
	typedef GetVarBlkPtr *GetVarBlkHandle;
#endif

/* ----------------------------------- */

typedef struct ConnPublic {
						void (*read)(void) ;
						void (*write)(void) ;
						void (*callBack)(void *) ;
						void *CallBackUserWorkSpace ;
						void (*YieldOpen)(void *) ;
						void *YieldUserWorkSpace ;
						int4 TimeBeforeYield ;
						int2 error ;
						} ConnPublic ;

typedef void 	*ConnectHandle;				/* semi-Private Handle structure on ConnPublic */
typedef void 	*NetworkComponentHandle;	/* private structure */
typedef Handle4D BufferHandle;				/* always a Handle but cast in void * for compatibility Mac/PC */
typedef Handle4D ArrayHandle;				/* always a Handle but cast in void * for compatibility Mac/PC */
typedef Handle4D FieldList_Plus_ArrayHandle;/* always a Handle but cast in void * for compatibility Mac/PC */
typedef Handle4D UserGroupHandle;			/* always a Handle but cast in void * for compatibility Mac/PC */
typedef int4 	*NetworkLocationPtr;


/* X-Platform API to allocate blocks of memory compatible with 4d open API		  */
/* Handle4D are not pointers to pointers, use Lock4DHandle to get an address      */

extern ST_PASCAL (Handle4D) Alloc4DHandle(int4 size);
extern ST_PASCAL (char) Resize4DHandle(Handle4D theHandle,int4 len);
extern ST_PASCAL (void) Free4DHandle(Handle4D theHandle);
extern ST_PASCAL (void *) Lock4DHandle (Handle4D theHandle);
extern ST_PASCAL (void) Unlock4DHandle (Handle4D theHandle);
extern ST_PASCAL (int4) SizeOf4DHandle(Handle4D theHandle);
						
/* ============================================================================== */

#if WINVER
extern ST_PASCAL (void) _4D_Init4DWS(void);
#else
extern ST_PASCAL (void) _4D_Init4DWS(const CFragInitBlock* theInitBlock);
#endif

extern ST_PASCAL (void) _4D_DeInit4DWS(void);

extern ST_PASCAL (NetworkComponentHandle) _4D_InitNetworkComponent(int2 ComponentNumber);

extern ST_PASCAL (void) _4D_DeInitNetworkComponent(int2 ComponentNumber, NetworkComponentHandle Data);

extern ST_PASCAL (int2) _4D_NB_NetworkComponent(void);

extern ST_PASCAL (int2) _4D_GetComponentInfo(int2 IndNumber, int2 *ComponentNumber, pstring ComponentName);

extern ST_PASCAL (int2) _4D_GetServerInfo(ConnectHandle CIDH, ServerInfoRec *ServerInfo);

/* ----------------------------------- */

extern ST_PASCAL (int2) _4D_Select4DServerWithOther(pstring Name, NetworkLocationPtr *NetworkLocation, int2 ComponentNumber,NetworkComponentHandle Data);

extern ST_PASCAL (int2) _4D_Select4DServer(pstring Name, NetworkLocationPtr *NetworkLocation, int2 ComponentNumber,NetworkComponentHandle Data);

extern ST_PASCAL (int2) _4D_Find4DServer(pstring Name, NetworkLocationPtr *NetworkLocation, int2 ComponentNumber,NetworkComponentHandle Data);
										
extern ST_PASCAL (int2) _4D_Open4DConnection(IDRec *ID, NetworkLocationPtr NetworkLocation, ConnectHandle *CIDH,int2 ComponentNumber, NetworkComponentHandle Data);

/*
	the opaque structure returned by _4D_Find4DServer, _4D_Select4DServer and _4D_Select4DServerWithOther
	needs to be deleted. Here is the appropriate call (m.c)
*/
extern ST_PASCAL(int2) _4D_FreeNetworkLocation(NetworkLocationPtr NetworkLocation,int2 ComponentNumber,NetworkComponentHandle Data);

extern ST_PASCAL (int2) _4D_Close4DConnection(ConnectHandle CIDH);

extern ST_PASCAL (int2) _4D_FlushServer(ConnectHandle CIDH);

/* ----------------------------------- */

extern ST_PASCAL (int2) _4D_SetConnectionAsync(ConnectHandle CIDH);
						 
extern ST_PASCAL (int2) _4D_SetConnectionSync(ConnectHandle CIDH);

extern ST_PASCAL (int2) _4D_Process4DRequest(ConnectHandle CIDH);

extern ST_PASCAL (int2) _4D_IsConnectionAsync(ConnectHandle CIDH,byte *AsyncMode);

/* ----------------------------------- */

extern ST_PASCAL (int2) _4D_EnableSystemTask(ConnectHandle CIDH);

extern ST_PASCAL (int2) _4D_DisableSystemTask(ConnectHandle CIDH);

extern ST_PASCAL (int2) _4D_IsEnableSystemTask(ConnectHandle CIDH,byte *EnableSystemTask);

/* ----------------------------------- */

extern ST_PASCAL (int2) _4D_SelectAllRecords(ConnectHandle CIDH, int2 Targetfile);

extern ST_PASCAL (int2) _4D_Search(ConnectHandle CIDH, SearchRecordPtr What, int4 *RecordsFound);

extern ST_PASCAL (int2) _4D_SearchInSelection(ConnectHandle CIDH, SearchRecordPtr What, int4 *RecordsFound);

extern ST_PASCAL (int2) _4D_Sort(ConnectHandle CIDH, SortRecordPtr What);

extern ST_PASCAL (int2) _4D_GotoSelectedRecord(ConnectHandle CIDH, int2 Targetfile, int4 RecNumberInSel);

extern ST_PASCAL (int2) _4D_GotoRecord(ConnectHandle CIDH, int2 Targetfile, int4 RecNumber);

extern ST_PASCAL (int2) _4D_RecordsInSelection(ConnectHandle CIDH, int2 Targetfile, int4 *Count);

extern ST_PASCAL( int2) _4D_CurrentRecordNumber(ConnectHandle CIDH, int2 Targetfile, int4 *RecNum, int4 *RecInSelection);

extern ST_PASCAL (int2) _4D_RecordsInFile(ConnectHandle CIDH, int2 Targetfile, int4 *Count);

extern ST_PASCAL (int2) _4D_AutoSequenceNumber(ConnectHandle CIDH,int2 Targetfile,int4 *AutoNumber);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_GetFields(ConnectHandle CIDH, ReqFieldRecPtr FieldList, BufferHandle *Buffer, byte *IsLocked);

extern ST_PASCAL ( int2) _4D_GetRelatedFields(ConnectHandle CIDH, ReqFieldRecPtr FieldList, BufferHandle *Buffer, byte *IsLocked);
					 
extern ST_PASCAL ( int2) _4D_GetRelatedFormattedFields(ConnectHandle CIDH, ReqFieldRecPtr FieldList, Handle4D Formats, BufferHandle *Buffer, byte *IsLocked);

extern ST_PASCAL ( int2) _4D_GetNthField(ConnectHandle CIDH,BufferHandle Buffer, int2 FieldNumber, DataRec *Field);

extern ST_PASCAL ( int2) _4D_UnlockCurrentRecord(ConnectHandle CIDH, int2 Targetfile);

extern ST_PASCAL ( BufferHandle) _4D_CreateBuffer(ConnectHandle CIDH,int2 Targetfile);

extern ST_PASCAL ( int2) _4D_AddToBuffer(ConnectHandle CIDH,BufferHandle Buffer, int2 FieldNumber, DataRec *Field);

extern ST_PASCAL ( int2) _4D_ModifyRecord(ConnectHandle CIDH, BufferHandle Buffer);

extern ST_PASCAL ( int2) _4D_NewRecord(ConnectHandle CIDH, BufferHandle Buffer);

extern ST_PASCAL ( int2) _4D_DeleteSelectedRecords(ConnectHandle CIDH, int2 Targetfile);

extern ST_PASCAL ( int2) _4D_DeleteCurrentRecord(ConnectHandle CIDH, int2 Targetfile);

extern ST_PASCAL ( int2) _4D_SetReadWriteMode(ConnectHandle CIDH, int2 Targetfile, byte ReadWrite);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_OneToMany_Join(ConnectHandle CIDH, int2 StartOfRelation_File, int2 StartOfRelation_Field);

extern ST_PASCAL ( int2) _4D_ManyToOne_Join(ConnectHandle CIDH, int2 StartOfRelation_File, int2 Targetfile);

extern ST_PASCAL ( int2) _4D_BuildDistinctValues(ConnectHandle CIDH, int2 TargetFile, int2 TargetField, ArrayHandle *BufferArray);

extern ST_PASCAL ( int2) _4D_ShrinkSelection(ConnectHandle CIDH,int2 Targetfile, int4 MaxRecords);

extern ST_PASCAL ( int2) _4D_GetFirst_Nth_Values(ConnectHandle CIDH, int2 TargetFile, int2 TargetField, int4 Maxrecords);

extern ST_PASCAL ( int2) _4D_SelectionToArray(ConnectHandle CIDH, ReqFieldRecPtr FieldList, ArrayHandle *BufferArray);

extern ST_PASCAL ( int2) _4D_SubSelectionToArray(ConnectHandle CIDH, ReqFieldRecPtr FieldList, 
																		int4 First, int4 Last, ArrayHandle *BufferArray);

extern ST_PASCAL ( int2) _4D_ArrayToSelection(ConnectHandle CIDH, FieldList_Plus_ArrayHandle BufferArray);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_Start4DTransaction(ConnectHandle CIDH);
						  
extern ST_PASCAL ( int2) _4D_Commit(ConnectHandle CIDH);

extern ST_PASCAL ( int2) _4D_RollBack(ConnectHandle CIDH);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_CopySelection(ConnectHandle CIDH, pstring SelectionName, int2 Targetfile);

extern ST_PASCAL ( int2) _4D_MoveSelection(ConnectHandle CIDH, pstring SelectionName, int2 Targetfile);

extern ST_PASCAL ( int2) _4D_UseSelection(ConnectHandle CIDH, pstring SelectionName, int2 *Targetfile);

extern ST_PASCAL ( int2) _4D_KillSelection(ConnectHandle CIDH, pstring SelectionName);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_CountFiles(ConnectHandle CIDH, int2 *Count);

extern ST_PASCAL ( int2) _4D_GetFileDefinition( ConnectHandle CIDH, int2 Targetfile, FileDefHandle *Fdef);

// ***
// optim specific to 4D Open for 4D: use its Cache mechanism for Selection To Array (m.c, 12/6/2003)
typedef int2 (*_4D_optim_GetFileDefinition)( ConnectHandle CIDH, int2 Targetfile, FileDefHandle *Fdef);
extern ST_PASCAL ( void) _4D_SetFileDefinitionCallback(_4D_optim_GetFileDefinition in);
// ***

extern ST_PASCAL ( int2) _4D_GetFieldNumbers(ConnectHandle CIDH, ReqFieldNamesPtr FieldNameList, ReqFieldRecHandle *FieldList);

extern ST_PASCAL ( int2) _4D_GetListProcess(ConnectHandle CIDH, ListProcessHandle *List);

extern ST_PASCAL ( int2) _4D_GetNbUsers(ConnectHandle CIDH, int2 *NbUsers);

extern ST_PASCAL ( int2) _4D_GetNbUserProcesses(ConnectHandle CIDH, int2 *NbUserProcesses);

extern ST_PASCAL ( int2) _4D_GetUsersAndGroups(ConnectHandle CIDH, int2 WhichSet, UserGroupHandle *Buffer, int2 *CountUsers, int2 *CountGroups);

extern ST_PASCAL ( void) _4D_GetNthUser(UserGroupHandle Buffer, int2 UserNumber, UserRec *User);

extern ST_PASCAL ( void) _4D_GetNthGroup(UserGroupHandle Buffer, int2 GroupNumber, GroupRecHandle *Group);

extern ST_PASCAL ( int2) _4D_GetFileList(ConnectHandle CIDH, FileListHandle *TheFileList);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_SetLogFile(ConnectHandle CIDH, pstring LogName);

extern ST_PASCAL ( int2) _4D_StartFullBackup(ConnectHandle CIDH);

extern ST_PASCAL ( int2) _4D_StartIncBackup(ConnectHandle CIDH);

extern ST_PASCAL ( int2) _4D_SendToBackupModule(ConnectHandle CIDH, Handle4D SentData, Handle4D *ReceivedData);

/* ----------------------------------- */

extern ST_PASCAL ( int2) _4D_SumField(ConnectHandle CIDH, int2 Targetfile, int2 Targetfield, Real *Result);

extern ST_PASCAL ( int2) _4D_AverageField(ConnectHandle CIDH, int2 Targetfile, int2 Targetfield, Real *Result);

extern ST_PASCAL ( int2) _4D_MaxField(ConnectHandle CIDH, int2 Targetfile, int2 Targetfield, Real *Result);

extern ST_PASCAL ( int2) _4D_MinField(ConnectHandle CIDH, int2 Targetfile, int2 Targetfield, Real *Result);

/* ----------------------------------- */
										
extern ST_PASCAL ( int2) _4D_Server_CurrentTime(ConnectHandle CIDH, int4*	ServerTime);

extern ST_PASCAL ( int2) _4D_Server_CurrentDate(ConnectHandle CIDH, Date4D*	ServerDate);

/* ----------------------------------- */
										
extern ST_PASCAL ( int2) _4D_WriteData(ConnectHandle CIDH, Handle4D Bufferdata);

extern ST_PASCAL ( int2) _4D_ReadData(ConnectHandle CIDH, reqdataPtr data, Handle4D *Array);

/* ----------------------------------- */
										
extern ST_PASCAL( void) _4D_StrPCToMac(ConnectHandle CIDH,unsigned char * String,int2 len);

extern ST_PASCAL( void) _4D_StrMacToPC(ConnectHandle CIDH,unsigned char * String,int2 len);

extern ST_PASCAL( int4) _4D_ConvertArray(ConnectHandle CIDH,int2 *TypeArray,ArrayHandle Array, int2 ConvertStringMacPC,int2 sensReal);

extern ST_PASCAL( void) _4D_ConvertField(ConnectHandle CIDH,DataRec *Field,int2 sensReal) ;

extern ST_PASCAL( int4) _4D_GetVersionNumber(void);

/* ----------------------------------- */

extern ST_PASCAL( int2) _4D_ProcessNumber(ConnectHandle	CIDH, st32 ProcessName, int2	*processNumber);

extern ST_PASCAL( int2) _4D_ExecuteOnServer(ConnectHandle	CIDH, ServerExecHandle	srvrExecHdl, int4	*processNumber);

extern ST_PASCAL( int2) _4D_GetProcessVar(ConnectHandle CIDH, GetVarBlkHandle varBlkHdl);

extern ST_PASCAL( int2) _4D_SetProcessVar(ConnectHandle	CIDH, SetVarBlkHandle varBlkHdl);

/* ----------------------------------- */

extern ST_PASCAL( int2) _4D_CreateEmptySet(ConnectHandle CIDH, st80 SetName, int2	Targetfile);

extern ST_PASCAL( int2) _4D_CreateSet(ConnectHandle CIDH, st80 SetName, int2 Targetfile);

extern ST_PASCAL( int2) _4D_UseSet(ConnectHandle CIDH, st80 SetName);

extern ST_PASCAL( int2) _4D_AddToSet(ConnectHandle CIDH, st80 SetName, int2	Targetfile);

extern ST_PASCAL( int2) _4D_RemoveFromSet(ConnectHandle CIDH, st80 SetName, int2	Targetfile);

extern ST_PASCAL( int2) _4D_ClearSet(ConnectHandle CIDH, st80 SetName);

extern ST_PASCAL( int2) _4D_IsInSet(ConnectHandle CIDH, st80 SetName, int4	recNum, int2 *IsIn);

extern ST_PASCAL( int2) _4D_RecordsInSet(ConnectHandle CIDH, st80 SetName, int4	*recInSet);

extern ST_PASCAL( int2) _4D_CopySet(ConnectHandle CIDH, st80 SetSourceName, st80 SetDestinationName);

extern ST_PASCAL( int2) _4D_UnionSet(ConnectHandle CIDH, st80 SetSourceOne, st80 SetSourceTwo, st80 SetDestinationName);

extern ST_PASCAL( int2) _4D_DifferenceSet(ConnectHandle CIDH, st80 SetSourceOne, st80 SetSourceTwo, st80 SetDestinationName);

extern ST_PASCAL( int2) _4D_IntersectionSet(ConnectHandle CIDH, st80 SetSourceOne, st80 SetSourceTwo, st80 SetDestinationName);

/* ----------------------------------- */

extern ST_PASCAL( int2) _4D_SetSemaphore(ConnectHandle	CIDH, st15	SemaphoreName, char	*Semaphore_State);

extern ST_PASCAL( int2) _4D_CheckSemaphore(ConnectHandle	CIDH, st15	SemaphoreName, char	*Semaphore_State);

extern ST_PASCAL( int2) _4D_ClearSemaphore(ConnectHandle	CIDH, st15 SemaphoreName);

#ifdef __cplusplus
	}
#endif



#if MACVER && OpenLib
#if __MWERKS__ //|| PRAGMA_ALIGN_SUPPORTED //CD 15/01/2001, unused now
#pragma options align=reset
#endif
#endif

#if WINVER
		#pragma pack()
#endif

//CD 07/06/2001
#if WINVER
HANDLE hLibInst;
#endif

#endif


